@@ -1,35 +1,23 @@
 <?php
 
-/*
- * This file is part of the Doctrine Bundle
- *
- * The code was originally distributed inside the Symfony framework.
- *
- * (c) Fabien Potencier <fabien@symfony.com>
- * (c) Doctrine Project, Benjamin Eberlei <kontakt@beberlei.de>
- *
- * For the full copyright and license information, please view the LICENSE
- * file that was distributed with this source code.
- */
-
 namespace Doctrine\Bundle\DoctrineBundle\Controller;
 
 use Doctrine\DBAL\Connection;
 use Doctrine\DBAL\Platforms\SQLServerPlatform;
+use Exception;
+use PDO;
 use Symfony\Component\DependencyInjection\ContainerAwareInterface;
 use Symfony\Component\DependencyInjection\ContainerInterface;
 use Symfony\Component\HttpFoundation\Response;
+use Symfony\Component\HttpKernel\Profiler\Profiler;
+use Symfony\Component\VarDumper\Cloner\Data;
 
 /**
  * ProfilerController.
- *
- * @author Christophe Coevoet <stof@notk.org>
  */
 class ProfilerController implements ContainerAwareInterface
 {
-    /**
-     * @var ContainerInterface
-     */
+    /** @var ContainerInterface */
     private $container;
 
     /**
@@ -43,31 +31,31 @@ class ProfilerController implements ContainerAwareInterface
     /**
      * Renders the profiler panel for the given token.
      *
-     * @param string  $token          The profiler token
-     * @param string  $connectionName
-     * @param integer $query
+     * @param string $token          The profiler token
+     * @param string $connectionName
+     * @param int    $query
      *
      * @return Response A Response instance
      */
     public function explainAction($token, $connectionName, $query)
     {
-        /** @var $profiler \Symfony\Component\HttpKernel\Profiler\Profiler */
+        /** @var Profiler $profiler */
         $profiler = $this->container->get('profiler');
         $profiler->disable();
 
         $profile = $profiler->loadProfile($token);
         $queries = $profile->getCollector('db')->getQueries();
 
-        if (!isset($queries[$connectionName][$query])) {
+        if (! isset($queries[$connectionName][$query])) {
             return new Response('This query does not exist.');
         }
 
         $query = $queries[$connectionName][$query];
-        if (!$query['explainable']) {
+        if (! $query['explainable']) {
             return new Response('This query cannot be explained.');
         }
 
-        /** @var $connection \Doctrine\DBAL\Connection */
+        /** @var Connection $connection */
         $connection = $this->container->get('doctrine')->getConnection($connectionName);
         try {
             if ($connection->getDatabasePlatform() instanceof SQLServerPlatform) {
@@ -75,14 +63,14 @@ class ProfilerController implements ContainerAwareInterface
             } else {
                 $results = $this->explainOtherPlatform($connection, $query);
             }
-        } catch (\Exception $e) {
+        } catch (Exception $e) {
             return new Response('This query cannot be explained.');
         }
 
-        return new Response($this->container->get('twig')->render('@Doctrine/Collector/explain.html.twig', array(
+        return new Response($this->container->get('twig')->render('@Doctrine/Collector/explain.html.twig', [
             'data' => $results,
             'query' => $query,
-        )));
+        ]));
     }
 
     private function explainSQLServerPlatform(Connection $connection, $query)
@@ -90,16 +78,29 @@ class ProfilerController implements ContainerAwareInterface
         if (stripos($query['sql'], 'SELECT') === 0) {
             $sql = 'SET STATISTICS PROFILE ON; ' . $query['sql'] . '; SET STATISTICS PROFILE OFF;';
         } else {
-            $sql = 'SET SHOWPLAN_TEXT ON; GO; SET NOEXEC ON; ' . $query['sql'] .'; SET NOEXEC OFF; GO; SET SHOWPLAN_TEXT OFF;';
+            $sql = 'SET SHOWPLAN_TEXT ON; GO; SET NOEXEC ON; ' . $query['sql'] . '; SET NOEXEC OFF; GO; SET SHOWPLAN_TEXT OFF;';
         }
-        $stmt = $connection->executeQuery($sql, $query['params'], $query['types']);
+
+        $params = $query['params'];
+
+        if ($params instanceof Data) {
+            $params = $params->getValue(true);
+        }
+
+        $stmt = $connection->executeQuery($sql, $params, $query['types']);
         $stmt->nextRowset();
-        return $stmt->fetchAll(\PDO::FETCH_ASSOC);
+        return $stmt->fetchAll(PDO::FETCH_ASSOC);
     }
 
     private function explainOtherPlatform(Connection $connection, $query)
     {
-        return $connection->executeQuery('EXPLAIN '.$query['sql'], $query['params'], $query['types'])
-            ->fetchAll(\PDO::FETCH_ASSOC);
+        $params = $query['params'];
+
+        if ($params instanceof Data) {
+            $params = $params->getValue(true);
+        }
+
+        return $connection->executeQuery('EXPLAIN ' . $query['sql'], $params, $query['types'])
+            ->fetchAll(PDO::FETCH_ASSOC);
     }
 }
