@@ -6,9 +6,12 @@ use Doctrine\Common\Persistence\ManagerRegistry;
 use Doctrine\ORM\Cache\Logging\CacheLoggerChain;
 use Doctrine\ORM\Cache\Logging\StatisticsCacheLogger;
 use Doctrine\ORM\Configuration;
+use Doctrine\ORM\EntityManager;
 use Doctrine\ORM\Mapping\ClassMetadataFactory;
+use Doctrine\ORM\Mapping\ClassMetadataInfo;
 use Doctrine\ORM\Tools\SchemaValidator;
 use Doctrine\ORM\Version;
+use Exception;
 use Symfony\Bridge\Doctrine\DataCollector\DoctrineDataCollector as BaseCollector;
 use Symfony\Component\HttpFoundation\Request;
 use Symfony\Component\HttpFoundation\Response;
@@ -37,7 +40,7 @@ class DoctrineDataCollector extends BaseCollector
     /**
      * {@inheritdoc}
      */
-    public function collect(Request $request, Response $response, \Exception $exception = null)
+    public function collect(Request $request, Response $response, Exception $exception = null)
     {
         parent::collect($request, $response, $exception);
 
@@ -58,13 +61,14 @@ class DoctrineDataCollector extends BaseCollector
             ],
         ];
 
+        /** @var EntityManager $em */
         foreach ($this->registry->getManagers() as $name => $em) {
             $entities[$name] = [];
             /** @var ClassMetadataFactory $factory */
             $factory   = $em->getMetadataFactory();
             $validator = new SchemaValidator($em);
 
-            /** @var $class \Doctrine\ORM\Mapping\ClassMetadataInfo */
+            /** @var ClassMetadataInfo $class */
             foreach ($factory->getLoadedMetadata() as $class) {
                 if (isset($entities[$name][$class->getName()])) {
                     continue;
@@ -224,11 +228,11 @@ class DoctrineDataCollector extends BaseCollector
                 $connectionGroupedQueries[$key]['count']++;
                 $totalExecutionMS += $query['executionMS'];
             }
-            usort($connectionGroupedQueries, function ($a, $b) {
+            usort($connectionGroupedQueries, static function ($a, $b) {
                 if ($a['executionMS'] === $b['executionMS']) {
                     return 0;
                 }
-                return ($a['executionMS'] < $b['executionMS']) ? 1 : -1;
+                return $a['executionMS'] < $b['executionMS'] ? 1 : -1;
             });
             $this->groupedQueries[$connection] = $connectionGroupedQueries;
         }
