@@ -4,6 +4,7 @@ namespace Doctrine\Bundle\DoctrineBundle\Repository;
 
 use Doctrine\Common\Persistence\ManagerRegistry;
 use Doctrine\ORM\EntityRepository;
+use LogicException;
 
 /**
  * Optional EntityRepository base class with a simplified constructor (for autowiring).
@@ -28,6 +29,13 @@ class ServiceEntityRepository extends EntityRepository implements ServiceEntityR
     {
         $manager = $registry->getManagerForClass($entityClass);
 
+        if ($manager === null) {
+            throw new LogicException(sprintf(
+                'Could not find the entity manager for class "%s". Check your Doctrine configuration to make sure it is configured to load this entity’s metadata.',
+                $entityClass
+            ));
+        }
+
         parent::__construct($manager, $manager->getClassMetadata($entityClass));
     }
 }
